package com.ecnu.platform.path

import com.ecnu.alg.pojo.path.data.AlgPathStrategyInputData
import com.ecnu.alg.pojo.path.inputs.AlgBaseMap
import com.ecnu.alg.pojo.path.inputs.AlgBaseMapArea
import com.ecnu.alg.pojo.path.inputs.AlgObstacle
import com.ecnu.alg.pojo.path.inputs.AlgParkSpace
import com.ecnu.alg.pojo.path.inputs.AlgVehicle
import com.ecnu.alg.pojo.path.inputs.AlgVehicleTask
import com.three.common.exception.ParamException
import com.three.common.utils.StringUtil
import com.three.data_api.dm.annos.BizApiMethod
import com.three.data_api.dm.annos.BizRequestPm
import com.three.data_api.dm.annos.Doc4MethodRes
import com.three.data_api.dm.constants.ThreeConstant
import com.three.redis.utils.RedisUtil
import org.springframework.beans.factory.annotation.Autowired

class PathStrategyDataService {

    public static final String AlgBaseMapKey = "path:AlgBaseMap:"
    public static final String AlgVehicleKey = "path:AlgVehicle:"
    public static final String AlgBaseMapAreaKey = "path:AlgBaseMapArea:"
    public static final String AlgObstacleKey = "path:AlgObstacle:"
    public static final String AlgParkSpaceKey = "path:AlgParkSpace:"
    public static final String AlgVehicleTaskKey = "path:AlgVehicleTask:"

    public static final String conn = "-"

    @Autowired
    private RedisUtil redisUtil;

//    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "初始化底图信息")
//    @Doc4MethodRes(key = "result", desc = "底图信息", refClasses = AlgBaseMap.class)
//    AlgBaseMap initBaseMap(@BizRequestPm(key = "id", nullable = false, desc = "底图ID") String id,
//                           @BizRequestPm(key = "name", nullable = false, desc = "底图名称") String name,
//                           @BizRequestPm(key = "pcsBorder", nullable = false, desc = "平面坐标边界") String pcsBorder) throws Exception {
//        AlgBaseMap algBaseMap = new AlgBaseMap()
//        algBaseMap.setId(id)
//        algBaseMap.setName(name)
//        algBaseMap.setPcsBorder(pcsBorder)
//        return algBaseMap
//    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "初始化底图信息")
    @Doc4MethodRes(key = "result", desc = "底图信息", refClasses = AlgBaseMap.class)
    AlgPathStrategyInputData initData(@BizRequestPm(key = "body", nullable = false, desc = "底图信息") AlgPathStrategyInputData algPathStrategyInputData) throws Exception {
        AlgBaseMap algBaseMap = algPathStrategyInputData.getAlgBaseMap();
        if (algBaseMap != null && StringUtil.isNotBlank(algBaseMap.getId())) {
            String mapId = algBaseMap.getId();
            redisUtil.set(AlgBaseMapKey + algBaseMap.getId(), algBaseMap)
            // 车辆信息
            for (AlgVehicle algVehicle : algPathStrategyInputData.getAlgVehicleList()) {
                redisUtil.set(AlgVehicleKey + mapId + conn + algVehicle.getVehicleNo(), algVehicle)
            }
            // 车辆任务信息
            for (AlgVehicleTask algVehicleTask : algPathStrategyInputData.getAlgVehicleTaskList()) {
                redisUtil.set(AlgVehicleTaskKey + mapId + conn + algVehicleTask.getVehicleNo(), algVehicleTask)
            }
            // 障碍物信息
            for (AlgObstacle algObstacle : algPathStrategyInputData.getAlgObstacleList()) {
                redisUtil.set(AlgObstacleKey + mapId + conn + algObstacle.getId(), algObstacle)
            }
            // 停车位信息
            for (AlgParkSpace algParkSpace : algPathStrategyInputData.getAlgParkSpaceList()) {
                redisUtil.set(AlgParkSpaceKey + mapId + conn + algParkSpace.getId(), algParkSpace)
            }
        } else {
            throw new ParamException("底图信息为空")
        }
        return algPathStrategyInputData
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "查询所有底图信息")
    @Doc4MethodRes(key = "result", desc = "底图信息", refClasses = AlgBaseMap.class)
    List<AlgBaseMap> queryAllBaseMap(Map<String, Object> map) throws Exception {
        redisUtil.getByPrefix(AlgBaseMapKey + "*") as List<AlgBaseMap>
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "查询所有底图要素")
    @Doc4MethodRes(key = "result", desc = "底图要素信息", refClasses = [AlgVehicle.class, AlgBaseMapArea.class, AlgObstacle.class, AlgParkSpace.class])
    Map<String, Object> queryAllElement(@BizRequestPm(key = "mapId", nullable = false, desc = "底图ID") String mapId) throws Exception {
        Map<String, Object> reMap = new HashMap<>()
        List<AlgVehicle> algVehicleList = redisUtil.getByPrefix(AlgVehicleKey + mapId + conn + "*") as List<AlgVehicle>
        List<AlgBaseMapArea> algBaseMapAreaList = redisUtil.getByPrefix(AlgBaseMapAreaKey + mapId + conn + "*") as List<AlgBaseMapArea>
        List<AlgObstacle> algObstacleList = redisUtil.getByPrefix(AlgObstacleKey + mapId + conn + "*") as List<AlgObstacle>
        List<AlgParkSpace> algParkSpaceList = redisUtil.getByPrefix(AlgParkSpaceKey + mapId + conn + "*") as List<AlgParkSpace>
        reMap.put("algVehicleList", algVehicleList)
        reMap.put("algBaseMapAreaList", algBaseMapAreaList)
        reMap.put("algObstacleList", algObstacleList)
        reMap.put("algParkSpaceList", algParkSpaceList)
        return reMap;
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "新增选取点")
    void createPoint(@BizRequestPm(key = "mapId", nullable = false, desc = "底图ID") String mapId,
                     @BizRequestPm(key = "position", nullable = false, desc = "坐标点") String position,
                     @BizRequestPm(key = "elementType", nullable = false, desc = "点类型") String elementType,
                     @BizRequestPm(key = "id", nullable = false, desc = "编号") String id) throws Exception {
        AlgVehicle algVehicle = new AlgVehicle();
        algVehicle.setMapId(mapId)
        algVehicle.setVehicleNo(id)
        algVehicle.setCurPosition(position)
        redisUtil.set(AlgVehicleKey + mapId + conn + id, algVehicle)
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "新增绘制矩形")
    void createRectangle(@BizRequestPm(key = "mapId", nullable = false, desc = "底图ID") String mapId,
                         @BizRequestPm(key = "position", nullable = false, desc = "矩形坐标") String position,
                         @BizRequestPm(key = "elementType", nullable = false, desc = "点类型") String elementType,
                         @BizRequestPm(key = "id", nullable = false, desc = "编号") String id,
                         @BizRequestPm(key = "name", desc = "名称") String name) throws Exception {
        if ("parkSpace".equals(elementType)) {
            AlgParkSpace algParkSpace = new AlgParkSpace();
            algParkSpace.setMapId(mapId)
            algParkSpace.setId(id)
            algParkSpace.setPcsBorder(position)
            redisUtil.set(AlgParkSpaceKey + mapId + conn + id, algParkSpace)
        } else if ("obstacle".equals(elementType)) {
            AlgObstacle algObstacle = new AlgObstacle();
            algObstacle.setMapId(mapId)
            algObstacle.setId(id)
            algObstacle.setName(name)
            algObstacle.setPcsBorder(position)
            redisUtil.set(AlgObstacleKey + mapId + conn + id, algObstacle)
        } else if ("baseMapArea".equals(elementType)) {
            AlgBaseMapArea algBaseMapArea = new AlgBaseMapArea();
            algBaseMapArea.setMapId(mapId)
            algBaseMapArea.setId(id)
            algBaseMapArea.setPcsBorder(position)
            redisUtil.set(AlgBaseMapAreaKey + mapId + conn + id, algBaseMapArea)
        } else {
            throw new ParamException("类型(" + elementType + ")暂不支持")
        }
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "新增绘制区域")
    void createRegion(@BizRequestPm(key = "mapId", nullable = false, desc = "底图ID") String mapId,
                      @BizRequestPm(key = "position", nullable = false, desc = "区域坐标") String position,
                      @BizRequestPm(key = "elementType", nullable = false, desc = "点类型") String elementType,
                      @BizRequestPm(key = "id", nullable = false, desc = "编号") String id,
                      @BizRequestPm(key = "name", desc = "名称") String name) throws Exception {
        if ("parkSpace".equals(elementType)) {
            AlgParkSpace algParkSpace = new AlgParkSpace();
            algParkSpace.setMapId(mapId)
            algParkSpace.setId(id)
            algParkSpace.setPcsBorder(position)
            redisUtil.set(AlgParkSpaceKey + mapId + conn + id, algParkSpace)
        } else if ("obstacle".equals(elementType)) {
            AlgObstacle algObstacle = new AlgObstacle();
            algObstacle.setMapId(mapId)
            algObstacle.setId(id)
            algObstacle.setName(name)
            algObstacle.setPcsBorder(position)
            redisUtil.set(AlgObstacleKey + mapId + conn + id, algObstacle)
        } else if ("baseMapArea".equals(elementType)) {
            AlgBaseMapArea algBaseMapArea = new AlgBaseMapArea();
            algBaseMapArea.setMapId(mapId)
            algBaseMapArea.setId(id)
            algBaseMapArea.setPcsBorder(position)
            redisUtil.set(AlgBaseMapAreaKey + mapId + conn + id, algBaseMapArea)
        } else {
            throw new ParamException("类型(" + elementType + ")暂不支持")
        }
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "删除元素")
    void deleteElement(@BizRequestPm(key = "mapId", nullable = false, desc = "底图ID") String mapId,
                       @BizRequestPm(key = "elementType", nullable = false, desc = "底图要素类型") String elementType,
                       @BizRequestPm(key = "id", nullable = false, desc = "编号") String id) throws Exception {
        if ("parkSpace".equals(elementType)) {
            redisUtil.del(AlgParkSpaceKey + mapId + conn + id)
        } else if ("obstacle".equals(elementType)) {
            redisUtil.del(AlgObstacleKey + mapId + conn + id)
        } else if ("baseMapArea".equals(elementType)) {
            redisUtil.del(AlgBaseMapAreaKey + mapId + conn + id)
        } else {
            throw new ParamException("类型(" + elementType + ")暂不支持")
        }
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "新增车辆任务")
    void createVehicleTask(@BizRequestPm(key = "mapId", nullable = false, desc = "底图ID") String mapId,
                           @BizRequestPm(key = "vehicleNo", nullable = false, desc = "车辆编号") String vehicleNo,
                           @BizRequestPm(key = "startPosition", nullable = false, desc = "起点位置") String startPosition,
                           @BizRequestPm(key = "endPosition", nullable = false, desc = "终点位置") String endPosition) throws Exception {
        AlgVehicleTask algVehicleTask = new AlgVehicleTask();
        algVehicleTask.setMapId(mapId)
        algVehicleTask.setVehicleNo(vehicleNo)
        algVehicleTask.setStartPosition(startPosition)
        algVehicleTask.setEndPosition(endPosition)
        redisUtil.set(AlgVehicleTaskKey + mapId + conn + vehicleNo, algVehicleTask)
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "查询车辆任务")
    @Doc4MethodRes(key = "result", desc = "底图信息", refClasses = AlgVehicleTask.class)
    AlgVehicleTask queryVehicleTask(@BizRequestPm(key = "mapId", nullable = false, desc = "底图ID") String mapId,
                                    @BizRequestPm(key = "vehicleNo", nullable = false, desc = "车辆编号") String vehicleNo) throws Exception {
        return redisUtil.get(AlgVehicleTaskKey + mapId + conn + vehicleNo) as AlgVehicleTask
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "清空数据")
    void deleteAll(@BizRequestPm(key = "mapId", nullable = false, desc = "底图ID") String mapId) throws Exception {
        redisUtil.getRedisTemplate().delete(redisUtil.getKeyByPrefix(AlgParkSpaceKey + mapId + conn + "*"))
        redisUtil.getRedisTemplate().delete(redisUtil.getKeyByPrefix(AlgObstacleKey + mapId + conn + "*"))
        redisUtil.getRedisTemplate().delete(redisUtil.getKeyByPrefix(AlgBaseMapAreaKey + mapId + conn + "*"))
        redisUtil.getRedisTemplate().delete(redisUtil.getKeyByPrefix(AlgVehicleKey + mapId + conn + "*"))
        redisUtil.getRedisTemplate().delete(redisUtil.getKeyByPrefix(AlgVehicleTaskKey + mapId + conn + "*"))
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "查询数据")
    AlgPathStrategyInputData queryAll(@BizRequestPm(key = "mapId", nullable = false, desc = "底图ID") String mapId) throws Exception {
        AlgPathStrategyInputData algPathStrategyInputData = new AlgPathStrategyInputData();
        AlgBaseMap algBaseMap = redisUtil.get(AlgBaseMapKey + mapId) as AlgBaseMap;
        List<AlgVehicle> algVehicleList = redisUtil.getByPrefix(AlgVehicleKey + mapId + conn + "*") as List<AlgVehicle>
        List<AlgVehicleTask> algVehicleTaskList = redisUtil.getByPrefix(AlgVehicleTaskKey + mapId + conn + "*") as List<AlgVehicleTask>
        List<AlgBaseMapArea> algBaseMapAreaList = redisUtil.getByPrefix(AlgBaseMapAreaKey + mapId + conn + "*") as List<AlgBaseMapArea>
        List<AlgObstacle> algObstacleList = redisUtil.getByPrefix(AlgObstacleKey + mapId + conn + "*") as List<AlgObstacle>
        List<AlgParkSpace> algParkSpaceList = redisUtil.getByPrefix(AlgParkSpaceKey + mapId + conn + "*") as List<AlgParkSpace>
        algPathStrategyInputData.setAlgBaseMap(algBaseMap)
        algPathStrategyInputData.setAlgVehicleList(algVehicleList)
        algPathStrategyInputData.setAlgVehicleTaskList(algVehicleTaskList)
        algPathStrategyInputData.setAlgBaseMapAreaList(algBaseMapAreaList)
        algPathStrategyInputData.setAlgObstacleList(algObstacleList)
        algPathStrategyInputData.setAlgParkSpaceList(algParkSpaceList)
        return algPathStrategyInputData
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "查询数据")
    Map<String, Object> queryAllTransTo(@BizRequestPm(key = "mapId", nullable = false, desc = "底图ID") String mapId) throws Exception {
        AlgPathStrategyInputData algPathStrategyInputData = queryAll(mapId)
        Map<String, Object> reMap = new HashMap<>()
        AlgBaseMap algBaseMap = algPathStrategyInputData.getAlgBaseMap()
        List<AlgObstacle> algObstacleList = algPathStrategyInputData.getAlgObstacleList()
        if (algBaseMap != null && algObstacleList != null) {
            // blocks
            String[] pcss = algBaseMap.getPcsBorder().split(";")
            int max = Integer.MIN_VALUE;
            for (String pcs : pcss) {
                String[] ps = pcs.split(",")
                for (String p : ps) {
                    int pp = (int) Double.parseDouble(p)
                    if (pp > max) {
                        max = pp;
                    }
                }
            }
            List<Map<String, Object>> blocks = new ArrayList<>()
            for (int i = 1; i <= max; i++) {
                for (int j = 1; j <= max; j++) {
                    Map<String, Object> blockMap = new HashMap<>()
                    blockMap.put("id", ((i - 1) * max + j) + "")
                    blockMap.put("x", i)
                    blockMap.put("y", j)
                    boolean fg = false;
                    for (AlgObstacle algObstacle : algObstacleList) {
                        // 解析坐标
                        int[] point = [i, j];
                        double[] polygon = parsePolygonString(algObstacle.getPcsBorder());
                        // 判断点是否在多边形内
                        if (isPointInPolygon(polygon, point[0], point[1])) {
                            fg = true;
                            break
                        }
                    }
                    blockMap.put("isObstacle", fg)
                    blocks.add(blockMap)
                }
            }
            reMap.put("blocks", blocks)
            // tasks
            List<Map<String, Object>> tasks = new ArrayList<>()
            List<AlgVehicleTask> algVehicleTaskList = algPathStrategyInputData.getAlgVehicleTaskList()
            for (int i = 1; i <= algVehicleTaskList.size(); i++) {
                AlgVehicleTask algVehicleTask = algVehicleTaskList.get(i - 1)
                Map<String, Object> taskMap = new HashMap<>()
                taskMap.put("id", algVehicleTask.getVehicleNo())
                String[] sp = algVehicleTask.getStartPosition().split(",")
                int fi = (int) Double.parseDouble(sp[0])
                int fj = (int) Double.parseDouble(sp[1])
                taskMap.put("from", ((fi - 1) * max + fj) + "")
                String[] ep = algVehicleTask.getStartPosition().split(",")
                int ti = (int) Double.parseDouble(ep[0])
                int tj = (int) Double.parseDouble(ep[1])
                taskMap.put("to", ((ti - 1) * max + tj) + "")
                taskMap.put("beginTime", i)
                tasks.add(taskMap)
            }
            reMap.put("tasks", tasks)
        }
        return reMap
    }

    // 解析点坐标字符串

    // 解析多边形顶点坐标字符串
    private static double[] parsePolygonString(String polygonStr) {
        String[] vertexStrs = polygonStr.split(";");
        double[] vertices = new double[vertexStrs.length * 2];
        for (int i = 0; i < vertexStrs.length; i++) {
            String[] coords = vertexStrs[i].split(",");
            vertices[i * 2] = Double.parseDouble(coords[0]);
            vertices[i * 2 + 1] = Double.parseDouble(coords[1]);
        }
        return vertices;
    }

    // 判断点是否在多边形内
    public static boolean isPointInPolygon(double[] polygon, int pointX, int pointY) {
        int n = (int) (polygon.length / 2);
        boolean c = false;
        for (int i = 0, j = n - 1; i < n; i++) {
            if ((polygon[2 * i + 1] > pointY) != (polygon[2 * j + 1] > pointY) &&
                    pointX < (polygon[2 * j] - polygon[2 * i]) * (pointY - polygon[2 * i + 1]) /
                    (polygon[2 * j + 1] - polygon[2 * i + 1]) + polygon[2 * i]) {
                c = !c;
            }
            j = i;
        }
        return c;
    }

}
