package com.ecnu.platform.port

import com.ecnu.alg.pojo.port.inputs.AlgBollard
import com.three.common.exception.ParamException
import com.three.common.utils.IdWorker
import com.three.common.utils.StringUtil
import com.three.common.vo.PageResult
import com.three.data_api.dm.annos.BizApiMethod
import com.three.data_api.dm.annos.Doc4MethodRes
import com.three.data_api.dm.constants.ThreeConstant
import com.three.data_api.dm.entity.AuqResult
import com.three.redis.utils.RedisUtil
import org.springframework.beans.factory.annotation.Autowired


class BollardService {

    public static final String AlgBollardKey = "port:AlgBollard:"

    @Autowired
    private RedisUtil redisUtil;

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "查询缆桩信息")
    @Doc4MethodRes(key = "result", desc = "缆桩信息", refClasses = AlgBollard.class)
    AuqResult query(Map<String, Object> map) throws Exception {
        List<AlgBollard> algBollardList = redisUtil.getByPrefix(AlgBollardKey + "*") as List<AlgBollard>
        AuqResult auqResult = new AuqResult();
        PageResult pageResult = new PageResult();
        pageResult.setData(algBollardList)
        auqResult.setResult(pageResult)
        return auqResult
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "新增缆桩信息")
    void create(Map<String, Object> map) throws Exception {
        // 先删除缆桩
        redisUtil.del(redisUtil.getKeyByPrefix(AlgBollardKey + "*"))
        for (int i = 1; i <= 40; i++) {
            AlgBollard algBollard = new AlgBollard()
            algBollard.setGkey(IdWorker.generateIdStr())
            algBollard.setId(String.format("%02d", i) + "#")
            algBollard.setLocation(50 + (i - 1) * 50)
            redisUtil.set(AlgBollardKey + algBollard.getGkey(), algBollard)
        }
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "删除缆桩信息")
    void delete(Map<String, Object> map) throws Exception {
        if (StringUtil.isBlankStr(map.get("gkey"))) {
            throw new ParamException("主键(gkey)不可以为空")
        }
        redisUtil.del(AlgBollardKey + map.get("gkey"))
    }
}
