package com.ecnu.platform.port

import com.ecnu.alg.pojo.port.inputs.AlgCranePool
import com.ecnu.alg.pojo.port.inputs.AlgVstHatchCover
import com.three.common.utils.GsonUtil
import com.three.common.vo.PageResult
import com.three.data_api.dm.annos.BizApiMethod
import com.three.data_api.dm.annos.Doc4MethodRes
import com.three.data_api.dm.constants.ThreeConstant
import com.three.data_api.dm.entity.AuqResult
import com.three.data_api.dm.entity.three.ThreeAuq
import com.three.data_api.dm.entity.three.ThreeFilterNode
import com.three.data_api.dm.entity.three.ThreeResultExtras
import com.three.redis.utils.RedisUtil
import org.springframework.beans.factory.annotation.Autowired

class CraneResourcePlanService {

    public static final String AlgCranePoolKey = "port:AlgCranePool:"

    @Autowired
    private RedisUtil redisUtil;

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "查询船舶桥机信息")
    @Doc4MethodRes(key = "result", desc = "船舶桥机信息", refClasses = AlgCranePool.class)
    AuqResult query(Map<String, Object> map) throws Exception {
        ThreeAuq threeAuq = ThreeAuq.genAuq(map)
        // 解析参数，获得船舶结构主键
        Map<String, Object> simpleFilter = threeAuq.getSimpleFilter();
        String tmlCvGkey = simpleFilter.get("tmlCvGkey")
        // 查询舱盖板信息
        List<AlgCranePool> algCranePoolList = redisUtil.get(AlgCranePoolKey + tmlCvGkey) as List<AlgCranePool>
        // 返回结果
        AuqResult auqResult = new AuqResult();
        PageResult pageResult = new PageResult();
        for (AlgCranePool algCranePool : algCranePoolList) {
            Map<String, Object> map1 = new LinkedHashMap<>()
            map1.put("gkey", null)
            map1.put("tmlCvGkey", algCranePool.getVesselVisitId())
            map1.put("craneId", algCranePool.getCraneId())
            map1.put("planStartTime", algCranePool.getWorkStartTime())
            map1.put("planEndTime", algCranePool.getWorkEndTime())
            pageResult.getData().add(map1)
        }
        pageResult.setPageIndex(1)
        pageResult.setPageSize(20)
        pageResult.setTotal(algCranePoolList.size())
        pageResult.setTotalPage(pageResult.getTotal() / pageResult.getPageSize() as long)
        auqResult.setResult(pageResult)
        String extras = "{\"allAuqs\":[],\"currentAuq\":{\"queryName\":\"系统默认查询\",\"entityName\":\"CraneResourcePlan\",\"simpleFilter\":{\"tmlCvGkey\":\"01HSWQAGD41DCQBE84J2YD8CHC\"},\"isFuzzy\":false,\"paraFields\":[],\"paraMap\":{},\"displayFields\":[\"tmlCvGkey\",\"craneId\",\"planStartTime\",\"planEndTime\",\"creator\",\"created\",\"changer\",\"changed\"],\"selectFields\":[\"gkey\",\"tmlCvGkey\",\"craneId\",\"planStartTime\",\"planEndTime\",\"creator\",\"created\",\"changer\",\"changed\",\"isDeleted\"],\"orderByFields\":[[\"changed\",\"DESC\"]],\"pageIndex\":1,\"pageSize\":20},\"fieldDefs\":{\"gkey\":{\"fieldName\":\"gkey\",\"fieldKey\":\"entity.CraneResourcePlan.gkey\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"tmlCvGkey\":{\"fieldName\":\"tmlCvGkey\",\"fieldKey\":\"entity.CraneResourcePlan.tmlCvGkey\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"craneId\":{\"fieldName\":\"craneId\",\"fieldKey\":\"entity.CraneResourcePlan.craneId\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":false},\"planStartTime\":{\"fieldName\":\"planStartTime\",\"fieldKey\":\"entity.CraneResourcePlan.planStartTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"planEndTime\":{\"fieldName\":\"planEndTime\",\"fieldKey\":\"entity.CraneResourcePlan.planEndTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"creator\":{\"fieldName\":\"creator\",\"fieldKey\":\"entity.CraneResourcePlan.creator\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"created\":{\"fieldName\":\"created\",\"fieldKey\":\"entity.CraneResourcePlan.created\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"changer\":{\"fieldName\":\"changer\",\"fieldKey\":\"entity.CraneResourcePlan.changer\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"changed\":{\"fieldName\":\"changed\",\"fieldKey\":\"entity.CraneResourcePlan.changed\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"isDeleted\":{\"fieldName\":\"isDeleted\",\"fieldKey\":\"entity.CraneResourcePlan.isDeleted\",\"fieldType\":\"Integer\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":true}},\"trans\":{\"entity.CraneResourcePlan.gkey\":\"主键\",\"entity.CraneResourcePlan.tmlCvGkey\":\"航次主键\",\"entity.CraneResourcePlan.craneId\":\"桥机编号\",\"entity.CraneResourcePlan.planStartTime\":\"计划开始时间\",\"entity.CraneResourcePlan.planEndTime\":\"计划完成时间\",\"entity.CraneResourcePlan.creator\":\"创建人\",\"entity.CraneResourcePlan.created\":\"创建时间\",\"entity.CraneResourcePlan.changer\":\"修改人\",\"entity.CraneResourcePlan.changed\":\"修改时间\",\"entity.CraneResourcePlan.isDeleted\":\"已删除\",\"entity.CraneResourcePlan.isDeleted.1\":\"是\",\"entity.CraneResourcePlan.isDeleted.0\":\"否\"}}"
        ThreeResultExtras threeResultExtras = GsonUtil.fromJsonObject(extras, ThreeResultExtras.class)
        auqResult.setExtras(threeResultExtras)
        return auqResult
    }
}
