package com.ecnu.platform.port

import com.ecnu.alg.pojo.port.inputs.AlgVstHatchCover
import com.ecnu.alg.pojo.port.results.AlgReCwpBlock
import com.three.common.utils.GsonUtil
import com.three.common.vo.PageResult
import com.three.data_api.dm.annos.BizApiMethod
import com.three.data_api.dm.annos.Doc4MethodRes
import com.three.data_api.dm.constants.ThreeConstant
import com.three.data_api.dm.entity.AuqResult
import com.three.data_api.dm.entity.three.ThreeAuq
import com.three.data_api.dm.entity.three.ThreeFilterNode
import com.three.data_api.dm.entity.three.ThreeResultExtras
import com.three.redis.utils.RedisUtil
import org.springframework.beans.factory.annotation.Autowired

class CwpBlockVoService {

    public static final String AlgReCwpBlockKey = "port:AlgReCwpBlock:"

    @Autowired
    private RedisUtil redisUtil;

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "查询作业块信息")
    @Doc4MethodRes(key = "result", desc = "作业块信息", refClasses = AlgReCwpBlock.class)
    AuqResult query(Map<String, Object> map) throws Exception {
        ThreeAuq threeAuq = ThreeAuq.genAuq(map)
        // 解析参数，获得船舶航次主键
        List<String> tmlCvGkeys = new ArrayList<>();
        List<ThreeFilterNode> nodeList = threeAuq.getAdvanceFilter().getNodeList()
        for (ThreeFilterNode node : nodeList) {
            if ("tmlCvGkey".equals(node.getFieldName())) {
                tmlCvGkeys = node.getValues()[0].getValue() as List<String>
            }
        }
        // 查询作业块信息
        List<AlgReCwpBlock> algReCwpBlocks = new ArrayList<>()
        for (String tmlCvGkey : tmlCvGkeys) {
            List<AlgReCwpBlock> algReCwpBlockList = redisUtil.get(AlgReCwpBlockKey + tmlCvGkey) as List<AlgReCwpBlock>
            if (algReCwpBlockList != null) {
                algReCwpBlocks.addAll(algReCwpBlockList)
            }
        }
        // 返回结果
        AuqResult auqResult = new AuqResult();
        PageResult pageResult = new PageResult();
        pageResult.setData(algReCwpBlocks)
        pageResult.setPageIndex(1)
        pageResult.setPageSize(1000)
        pageResult.setTotal(algReCwpBlocks.size())
        pageResult.setTotalPage(pageResult.getTotal() / pageResult.getPageSize() as long)
        auqResult.setResult(pageResult)
        String extras = "{\"allAuqs\":[],\"currentAuq\":{\"queryName\":\"系统默认查询\",\"entityName\":\"CwpBlockVo\",\"simpleFilter\":{},\"isFuzzy\":false,\"advanceFilter\":{\"entityName\":\"CwpBlockVo\",\"nodeList\":[{\"nodeId\":\"01J2WCHXP74X1CC1TREGBWX6MA\",\"type\":\"CLAUSE\",\"fieldName\":\"tmlCvGkey\",\"operator\":\"IN\",\"values\":[{\"type\":\"DATA\",\"value\":[\"01HSWPH6D1SYC8NVMA8X8XFV4T\",\"01HSTQ88EGG3TN21P2XHK05GJ8\"]}]}]},\"paraFields\":[],\"paraMap\":{},\"displayFields\":[\"cwpGkey\",\"tmlCvGkey\",\"planStatus\",\"id\",\"type\",\"craneId\",\"cranePosition\",\"hatchNo\",\"bayNo\",\"planAmount\",\"sentAmount\",\"ldFlag\",\"craneSeq\",\"hatchSeq\",\"planStartTime\",\"planEndTime\",\"message\",\"selectReason\",\"creator\",\"created\",\"changer\",\"changed\"],\"selectFields\":[\"gkey\",\"cwpGkey\",\"tmlCvGkey\",\"planStatus\",\"id\",\"type\",\"craneId\",\"cranePosition\",\"hatchNo\",\"bayNo\",\"planAmount\",\"sentAmount\",\"ldFlag\",\"craneSeq\",\"hatchSeq\",\"planStartTime\",\"planEndTime\",\"message\",\"selectReason\",\"creator\",\"created\",\"changer\",\"changed\",\"isDeleted\"],\"orderByFields\":[[\"changed\",\"DESC\"]],\"pageIndex\":1,\"pageSize\":20},\"fieldDefs\":{\"gkey\":{\"fieldName\":\"gkey\",\"fieldKey\":\"entity.CwpBlockVo.gkey\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"cwpGkey\":{\"fieldName\":\"cwpGkey\",\"fieldKey\":\"entity.CwpBlockVo.cwpGkey\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"tmlCvGkey\":{\"fieldName\":\"tmlCvGkey\",\"fieldKey\":\"entity.CwpBlockVo.tmlCvGkey\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"planStatus\":{\"fieldName\":\"planStatus\",\"fieldKey\":\"entity.CwpBlockVo.planStatus\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":true},\"id\":{\"fieldName\":\"id\",\"fieldKey\":\"entity.CwpBlockVo.id\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"type\":{\"fieldName\":\"type\",\"fieldKey\":\"entity.CwpBlockVo.type\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":true},\"craneId\":{\"fieldName\":\"craneId\",\"fieldKey\":\"entity.CwpBlockVo.craneId\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"cranePosition\":{\"fieldName\":\"cranePosition\",\"fieldKey\":\"entity.CwpBlockVo.cranePosition\",\"fieldType\":\"Double\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hatchNo\":{\"fieldName\":\"hatchNo\",\"fieldKey\":\"entity.CwpBlockVo.hatchNo\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"bayNo\":{\"fieldName\":\"bayNo\",\"fieldKey\":\"entity.CwpBlockVo.bayNo\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"planAmount\":{\"fieldName\":\"planAmount\",\"fieldKey\":\"entity.CwpBlockVo.planAmount\",\"fieldType\":\"Double\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"sentAmount\":{\"fieldName\":\"sentAmount\",\"fieldKey\":\"entity.CwpBlockVo.sentAmount\",\"fieldType\":\"Double\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"ldFlag\":{\"fieldName\":\"ldFlag\",\"fieldKey\":\"entity.CwpBlockVo.ldFlag\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":true},\"craneSeq\":{\"fieldName\":\"craneSeq\",\"fieldKey\":\"entity.CwpBlockVo.craneSeq\",\"fieldType\":\"Integer\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hatchSeq\":{\"fieldName\":\"hatchSeq\",\"fieldKey\":\"entity.CwpBlockVo.hatchSeq\",\"fieldType\":\"Integer\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"planStartTime\":{\"fieldName\":\"planStartTime\",\"fieldKey\":\"entity.CwpBlockVo.planStartTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"planEndTime\":{\"fieldName\":\"planEndTime\",\"fieldKey\":\"entity.CwpBlockVo.planEndTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"message\":{\"fieldName\":\"message\",\"fieldKey\":\"entity.CwpBlockVo.message\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"selectReason\":{\"fieldName\":\"selectReason\",\"fieldKey\":\"entity.CwpBlockVo.selectReason\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"creator\":{\"fieldName\":\"creator\",\"fieldKey\":\"entity.CwpBlockVo.creator\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"created\":{\"fieldName\":\"created\",\"fieldKey\":\"entity.CwpBlockVo.created\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"changer\":{\"fieldName\":\"changer\",\"fieldKey\":\"entity.CwpBlockVo.changer\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"changed\":{\"fieldName\":\"changed\",\"fieldKey\":\"entity.CwpBlockVo.changed\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"isDeleted\":{\"fieldName\":\"isDeleted\",\"fieldKey\":\"entity.CwpBlockVo.isDeleted\",\"fieldType\":\"Integer\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":true}},\"trans\":{\"entity.CwpBlockVo.gkey\":\"主键\",\"entity.CwpBlockVo.cwpGkey\":\"计划主键\",\"entity.CwpBlockVo.tmlCvGkey\":\"航次主键\",\"entity.CwpBlockVo.planStatus\":\"计划状态\",\"entity.CwpBlockVo.planStatus.ACTIVE\":\"有效\",\"entity.CwpBlockVo.planStatus.DEACTIVE\":\"无效\",\"entity.CwpBlockVo.id\":\"编号\",\"entity.CwpBlockVo.type\":\"作业块类型\",\"entity.CwpBlockVo.type.CNT\":\"集装箱\",\"entity.CwpBlockVo.type.HC\":\"舱盖板\",\"entity.CwpBlockVo.type.END\":\"清舱\",\"entity.CwpBlockVo.craneId\":\"桥机编号\",\"entity.CwpBlockVo.cranePosition\":\"桥机作业位置\",\"entity.CwpBlockVo.hatchNo\":\"舱编号\",\"entity.CwpBlockVo.bayNo\":\"倍位编号\",\"entity.CwpBlockVo.planAmount\":\"计划作业量\",\"entity.CwpBlockVo.sentAmount\":\"已发送作业量\",\"entity.CwpBlockVo.ldFlag\":\"装卸船标志\",\"entity.CwpBlockVo.ldFlag.L\":\"装船\",\"entity.CwpBlockVo.ldFlag.D\":\"卸船\",\"entity.CwpBlockVo.ldFlag.LD\":\"边装边卸\",\"entity.CwpBlockVo.craneSeq\":\"桥机序\",\"entity.CwpBlockVo.hatchSeq\":\"舱序\",\"entity.CwpBlockVo.planStartTime\":\"计划开始时间\",\"entity.CwpBlockVo.planEndTime\":\"计划完成时间\",\"entity.CwpBlockVo.message\":\"作业块提示消息\",\"entity.CwpBlockVo.selectReason\":\"选择作业原因\",\"entity.CwpBlockVo.creator\":\"创建人\",\"entity.CwpBlockVo.created\":\"创建时间\",\"entity.CwpBlockVo.changer\":\"修改人\",\"entity.CwpBlockVo.changed\":\"修改时间\",\"entity.CwpBlockVo.isDeleted\":\"已删除\",\"entity.CwpBlockVo.isDeleted.1\":\"是\",\"entity.CwpBlockVo.isDeleted.0\":\"否\"}}"
        ThreeResultExtras threeResultExtras = GsonUtil.fromJsonObject(extras, ThreeResultExtras.class)
        auqResult.setExtras(threeResultExtras)
        return auqResult
    }
}
