package com.ecnu.platform.port

import com.ecnu.alg.pojo.port.inputs.AlgVstHatchCover
import com.three.common.utils.GsonUtil
import com.three.common.vo.PageResult
import com.three.data_api.dm.annos.BizApiMethod
import com.three.data_api.dm.annos.Doc4MethodRes
import com.three.data_api.dm.constants.ThreeConstant
import com.three.data_api.dm.entity.AuqResult
import com.three.data_api.dm.entity.three.ThreeAuq
import com.three.data_api.dm.entity.three.ThreeFilterNode
import com.three.data_api.dm.entity.three.ThreeResultExtras
import com.three.redis.utils.RedisUtil
import org.springframework.beans.factory.annotation.Autowired

class VstHatchCoverService {

    public static final String AlgVstHatchCoverKey = "port:AlgVstHatchCover:"

    @Autowired
    private RedisUtil redisUtil;

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "查询舱盖板信息")
    @Doc4MethodRes(key = "result", desc = "舱盖板信息", refClasses = AlgVstHatchCover.class)
    AuqResult query(Map<String, Object> map) throws Exception {
        ThreeAuq threeAuq = ThreeAuq.genAuq(map)
        // 解析参数，获得船舶结构主键
        Map<String, Object> simpleFilter = threeAuq.getSimpleFilter();
        String vesselStructGkey = simpleFilter.get("vesselStructGkey")
        // 查询舱盖板信息
        List<AlgVstHatchCover> algVstHatchCoverList = redisUtil.getByPrefix(AlgVstHatchCoverKey + vesselStructGkey + ":*") as List<AlgVstHatchCover>
        // 解析参数，获得舱位主键
        List<String> hatchGkeys = new ArrayList<>();
        List<ThreeFilterNode> nodeList = threeAuq.getAdvanceFilter().getNodeList()
        for (ThreeFilterNode node : nodeList) {
            if ("hatchGkey".equals(node.getFieldName())) {
                hatchGkeys = node.getValues()[0].getValue() as List<String>
            }
        }
        List<AlgVstHatchCover> algVstHatchCoverList2 = new ArrayList<>();
        for (AlgVstHatchCover algVstHatchCover : algVstHatchCoverList) {
            if (hatchGkeys.contains(algVstHatchCover.getHatchGkey())) {
                algVstHatchCoverList2.add(algVstHatchCover)
            }
        }
        AuqResult auqResult = new AuqResult();
        PageResult pageResult = new PageResult();
        pageResult.setData(algVstHatchCoverList2)
        pageResult.setPageIndex(1)
        pageResult.setPageSize(20)
        pageResult.setTotal(algVstHatchCoverList2.size())
        pageResult.setTotalPage(pageResult.getTotal() / pageResult.getPageSize() as long)
        auqResult.setResult(pageResult)
        String extras = "{\"allAuqs\":[],\"currentAuq\":{\"queryName\":\"系统默认查询\",\"entityName\":\"VstHatchCover\",\"simpleFilter\":{\"vesselStructGkey\":\"PRESTIGE\"},\"isFuzzy\":false,\"advanceFilter\":{\"entityName\":\"VstHatchCover\",\"nodeList\":[{\"nodeId\":\"01J2WA8K10RPZ082QT7G0SQKH0\",\"type\":\"CLAUSE\",\"fieldName\":\"hatchGkey\",\"operator\":\"IN\",\"values\":[{\"type\":\"DATA\",\"value\":[\"56952\",\"56953\"]}]}]},\"paraFields\":[],\"paraMap\":{},\"displayFields\":[\"vesselStructGkey\",\"hatchGkey\",\"bayNo\",\"openSeq\",\"hatchFromRowNo\",\"hatchToRowNo\",\"deckFromRowNo\",\"deckToRowNo\",\"weightMt\",\"creator\",\"created\",\"changer\",\"changed\"],\"selectFields\":[\"gkey\",\"vesselStructGkey\",\"hatchGkey\",\"bayNo\",\"openSeq\",\"hatchFromRowNo\",\"hatchToRowNo\",\"deckFromRowNo\",\"deckToRowNo\",\"weightMt\",\"creator\",\"created\",\"changer\",\"changed\",\"isDeleted\"],\"orderByFields\":[[\"changed\",\"DESC\"]],\"pageIndex\":1,\"pageSize\":20},\"fieldDefs\":{\"gkey\":{\"fieldName\":\"gkey\",\"fieldKey\":\"entity.VstHatchCover.gkey\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"vesselStructGkey\":{\"fieldName\":\"vesselStructGkey\",\"fieldKey\":\"entity.VstHatchCover.vesselStructGkey\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hatchGkey\":{\"fieldName\":\"hatchGkey\",\"fieldKey\":\"entity.VstHatchCover.hatchGkey\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"bayNo\":{\"fieldName\":\"bayNo\",\"fieldKey\":\"entity.VstHatchCover.bayNo\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"openSeq\":{\"fieldName\":\"openSeq\",\"fieldKey\":\"entity.VstHatchCover.openSeq\",\"fieldType\":\"Integer\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hatchFromRowNo\":{\"fieldName\":\"hatchFromRowNo\",\"fieldKey\":\"entity.VstHatchCover.hatchFromRowNo\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hatchToRowNo\":{\"fieldName\":\"hatchToRowNo\",\"fieldKey\":\"entity.VstHatchCover.hatchToRowNo\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"deckFromRowNo\":{\"fieldName\":\"deckFromRowNo\",\"fieldKey\":\"entity.VstHatchCover.deckFromRowNo\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"deckToRowNo\":{\"fieldName\":\"deckToRowNo\",\"fieldKey\":\"entity.VstHatchCover.deckToRowNo\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"weightMt\":{\"fieldName\":\"weightMt\",\"fieldKey\":\"entity.VstHatchCover.weightMt\",\"fieldType\":\"Double\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"creator\":{\"fieldName\":\"creator\",\"fieldKey\":\"entity.VstHatchCover.creator\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"created\":{\"fieldName\":\"created\",\"fieldKey\":\"entity.VstHatchCover.created\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"changer\":{\"fieldName\":\"changer\",\"fieldKey\":\"entity.VstHatchCover.changer\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"changed\":{\"fieldName\":\"changed\",\"fieldKey\":\"entity.VstHatchCover.changed\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"isDeleted\":{\"fieldName\":\"isDeleted\",\"fieldKey\":\"entity.VstHatchCover.isDeleted\",\"fieldType\":\"Integer\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":true}},\"trans\":{\"entity.VstHatchCover.gkey\":\"主键\",\"entity.VstHatchCover.vesselStructGkey\":\"船舶结构主键\",\"entity.VstHatchCover.hatchGkey\":\"船舱主键\",\"entity.VstHatchCover.bayNo\":\"贝号\",\"entity.VstHatchCover.openSeq\":\"开仓顺序\",\"entity.VstHatchCover.hatchFromRowNo\":\"舱内开始排号\",\"entity.VstHatchCover.hatchToRowNo\":\"舱内结束排号\",\"entity.VstHatchCover.deckFromRowNo\":\"甲板上开始排号\",\"entity.VstHatchCover.deckToRowNo\":\"甲板上结束排号\",\"entity.VstHatchCover.weightMt\":\"舱盖板重量\",\"entity.VstHatchCover.creator\":\"创建人\",\"entity.VstHatchCover.created\":\"创建时间\",\"entity.VstHatchCover.changer\":\"修改人\",\"entity.VstHatchCover.changed\":\"修改时间\",\"entity.VstHatchCover.isDeleted\":\"已删除\",\"entity.VstHatchCover.isDeleted.1\":\"是\",\"entity.VstHatchCover.isDeleted.0\":\"否\"}}"
        ThreeResultExtras threeResultExtras = GsonUtil.fromJsonObject(extras, ThreeResultExtras.class)
        auqResult.setExtras(threeResultExtras)
        return auqResult
    }
}
