package com.ecnu.platform.service

import com.ecnu.alg.pojo.path.data.AlgPathStrategyInputData
import com.three.common.exception.ParamException
import com.three.common.utils.GroovyUtil
import com.three.common.utils.GsonUtil
import com.three.common.utils.StringUtil
import com.three.common.vo.SelectOption
import com.three.data_api.dm.annos.BizApiMethod
import com.three.data_api.dm.annos.BizRequestPm
import com.three.data_api.dm.annos.Doc4MethodRes
import com.three.data_api.dm.constants.ParamName
import com.three.data_api.dm.constants.ThreeConstant
import com.three.data_api.dm.entity.AuqResult
import com.three.data_api.dm.entity.three.ThreeAuq
import com.three.data_api.dm.service.DaoService
import com.three.data_api.service.impl.BaseEntityServiceImpl
import org.springframework.beans.factory.annotation.Autowired

class AlgExecutionLogService extends BaseEntityServiceImpl {

    public static final String EN_AlgExecutionLog = "AlgExecutionLog"
    public static final String EN_AlgExeLogVo = "AlgExeLogVo"

    @Autowired
    private DaoService daoService;

    @Override
    String getEntityId(Map<String, Object> map) {
        return EN_AlgExecutionLog
    }

    @Override
    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "查询算法执行日志")
    @Doc4MethodRes(key = "result", desc = "算法执行日志", refClassName = EN_AlgExecutionLog)
    AuqResult query(Map<String, Object> map) throws Exception {
        if (StringUtil.isBlank(map.get(ParamName.pageIndex)) || StringUtil.isBlank(map.get(ParamName.pageSize))) {
            throw new ParamException("分页参数(" + ParamName.pageSize + "," + ParamName.pageIndex + ")不可以为空")
        }
        if (StringUtil.isNotBlankStr(map.get("versionGkey"))) {
            if (map.get("simpleFilter") == null) {
                map.put("simpleFilter", new HashMap())
            }
            Map<String, Object> simpleFilter = map.get("simpleFilter") as Map<String, Object>;
            simpleFilter.put("versionGkey", map.get("versionGkey"))
        }
        return daoService.findAuq(EN_AlgExeLogVo, map)
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "查询算法执行日志详情")
    @Doc4MethodRes(key = "result", desc = "算法执行日志", refClassName = EN_AlgExecutionLog)
    Object queryOne(@BizRequestPm(key = "gkey", nullable = false, desc = "算法执行日志主键") String gkey) throws Exception {
        return findOneObject(EN_AlgExecutionLog, gkey)
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "结果展示")
    @Doc4MethodRes(key = "result", desc = "算法执行日志", refClassName = EN_AlgExecutionLog)
    Object showResult(@BizRequestPm(key = "gkey", nullable = false, desc = "算法执行日志主键") String gkey) throws Exception {
        Map<String, Object> resultMap = new HashMap<>()
        // 根据日志主键查询日志记录，返回输入数据、算法结果
        Object object = findOneObject(EN_AlgExecutionLog, gkey)

        return object
    }

    @Override
    List<SelectOption> getSelectOptions(Map<String, Object> map) throws Exception {
        return super.getSelectOptions(map)
    }

    @Override
    List<ThreeAuq> getAllAuqs(Map<String, Object> map) throws Exception {
        return super.getAllAuqs(map)
    }

    @Override
    void create(Map<String, Object> map) throws Exception {
        super.create(map)
    }

    @Override
    void update(Map<String, Object> map) throws Exception {
        super.update(map)
    }

    @Override
    void delete(Map<String, Object> map) throws Exception {
        super.delete(map)
    }

    @Override
    void recover(Map<String, Object> map) throws Exception {
        super.recover(map)
    }
}
