package com.ecnu.platform.service

import com.three.common.exception.ParamException
import com.three.common.utils.GroovyUtil
import com.three.common.utils.StringUtil
import com.three.common.vo.SelectOption
import com.three.data_api.dm.annos.BizApiMethod
import com.three.data_api.dm.annos.BizRequestPm
import com.three.data_api.dm.annos.Doc4MethodRes
import com.three.data_api.dm.constants.ParamName
import com.three.data_api.dm.constants.ThreeConstant
import com.three.data_api.dm.entity.AuqResult
import com.three.data_api.dm.entity.three.ThreeAuq
import com.three.data_api.service.impl.BaseEntityServiceImpl

class AlgVersionVoService extends BaseEntityServiceImpl {

    public static final String EN_AlgVersion = "AlgVersion"
    public static final String EN_AlgVersionVo = "AlgVersionVo"

    @Override
    String getEntityId(Map<String, Object> map) {
        return EN_AlgVersionVo
    }

    @Override
    AuqResult query(Map<String, Object> map) throws Exception {
        map.put(ParamName.resource, ParamName.resource)
        return super.query(map)
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "查询算法版本")
    @Doc4MethodRes(key = "result", desc = "算法版本", refClassName = EN_AlgVersionVo)
    AuqResult queryByAlg(@BizRequestPm(key = "algGkey", nullable = false, desc = "算法主键") String algGkey) throws Exception {
        if (StringUtil.isBlank(algGkey)) {
            throw new ParamException("算法主键(algGkey)不可以为空")
        }
        Map<String, Object> map = new HashMap<>()
        Map<String, Object> simpleFilter = new HashMap<>();
        simpleFilter.put("algGkey", algGkey)
        map.put("simpleFilter", simpleFilter)
        map.put(ParamName.resource, ParamName.resource)
        return daoService.findAuq(EN_AlgVersionVo, map)
    }

    @Override
    List<SelectOption> getSelectOptions(Map<String, Object> map) throws Exception {
        return super.getSelectOptions(map)
    }

    @Override
    List<ThreeAuq> getAllAuqs(Map<String, Object> map) throws Exception {
        return super.getAllAuqs(map)
    }

    @Override
    void create(Map<String, Object> map) throws Exception {
        // 获取入参值Map
        Map paras = (map.get("paras") as List<Map>)[0]
        if (StringUtil.isNotBlank(paras.get("isOnline")) && 1== paras.get("isOnline")) {
            checkOnline(paras.get("algGkey") as String, null)
        }
        super.create(map)
    }

    @Override
    void update(Map<String, Object> map) throws Exception {
        // 获取入参值Map
        Map paras = (map.get("paras") as List<Map>)[0]
        if (StringUtil.isNotBlank(paras.get("isOnline")) && 1== paras.get("isOnline")) {
            checkOnline(paras.get("algGkey") as String, paras.get("gkey") as String)
        }
        super.update(map)
    }

    @Override
    void delete(Map<String, Object> map) throws Exception {
        super.delete(map)
    }

    @Override
    void recover(Map<String, Object> map) throws Exception {
        super.recover(map)
    }

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "修改算法在线状态")
    void updateOnline(@BizRequestPm(key = "gkey", nullable = false, desc = "算法版本主键") String gkey) throws Exception {
        Object object = findOneObject(EN_AlgVersion, gkey)
        // 如果将是否在线状态设置成1，则需要校验该算法是否已经存在其他第三方的算法在线
        int flag = (int) GroovyUtil.getProperty(object, "isOnline")
        if (flag == 0) {
            checkOnline(GroovyUtil.getProperty(object, "algGkey") as String, null)
            flag = 1
        } else {
            flag = 0
        }
        Map<String, Object> valueMap = new HashMap<>()
        valueMap.put("isOnline", flag)
        daoService.updateMetaEntityByGkey(EN_AlgVersion, valueMap, gkey)
    }

    private checkOnline(String algGkey, String gkey) {
        Map whereValueMap = new HashMap()
        whereValueMap.put("algGkey", algGkey)
        String sql = "select gkey from ALG_VERSION where IS_DELETED = 0 and ALG_GKEY = :algGkey and IS_ONLINE = 1"
        if (StringUtil.isNotBlank(gkey)) {
            sql += " and gkey != :gkey"
            whereValueMap.put("gkey", gkey)
        }
        List<Object> objectList = daoService.findListWithSql(EN_AlgVersion, sql, null, whereValueMap)
        if (objectList.size() > 0) {
            throw new ParamException("已经存在其他第三方算法在线")
        }
    }
}
