package com.ecnu.platform.port

import com.ecnu.alg.pojo.port.inputs.AlgVesselVisit
import com.three.common.utils.GsonUtil
import com.three.common.vo.PageResult
import com.three.data_api.dm.annos.BizApiMethod
import com.three.data_api.dm.annos.Doc4MethodRes
import com.three.data_api.dm.constants.ThreeConstant
import com.three.data_api.dm.entity.AuqResult
import com.three.data_api.dm.entity.three.ThreeResultExtras
import com.three.redis.utils.RedisUtil
import org.springframework.beans.factory.annotation.Autowired

class TmlVesselVisitVoService {

    public static final String AlgVesselVisitKey = "port:AlgVesselVisit:"

    @Autowired
    private RedisUtil redisUtil;

    @BizApiMethod(matchedApis = ThreeConstant.BIZ_API_NAME, desc = "查询船舶航次信息")
    @Doc4MethodRes(key = "result", desc = "船舶航次信息", refClasses = AlgVesselVisit.class)
    AuqResult queryVessel(Map<String, Object> map) throws Exception {
        // 根据入参查询对应的船舶航次
        List<AlgVesselVisit> algVesselVisitList = redisUtil.getByPrefix(AlgVesselVisitKey + "*") as List<AlgVesselVisit>
        AuqResult auqResult = new AuqResult();
        PageResult pageResult = new PageResult();
        pageResult.setPageIndex(1)
        pageResult.setPageSize(20)
        for (AlgVesselVisit algVesselVisit : algVesselVisitList) {
            Map visitMap = new LinkedHashMap<>()
            visitMap.put("gkey", algVesselVisit.getVesselVisitId())
            visitMap.put("id", algVesselVisit.getVesselVisitId())
            visitMap.put("phase", "V_ON_BERTH")
            visitMap.put("hbrCvGkey", null)
            visitMap.put("actStartBollardId", algVesselVisit.getStartBollardId())
            visitMap.put("actEndBollardId", algVesselVisit.getEndBollardId())
            visitMap.put("actDirection", algVesselVisit.getBerthDirection())
            visitMap.put("carId", algVesselVisit.getVesselCode())
            visitMap.put("carNameLocal", algVesselVisit.getVesselName())
            visitMap.put("carVesselStructGkey", algVesselVisit.getVesselCode())
            pageResult.getData().add(visitMap)
        }
        pageResult.setTotal(algVesselVisitList.size())
        pageResult.setTotalPage(pageResult.getTotal() / pageResult.getPageSize() as long)
        auqResult.setResult(pageResult)
        String extras = "{\"allAuqs\":[{\"gkey\":\"01HTME1M9K8KHBFGZHVX0145WD\",\"queryName\":\"test\",\"description\":\"test\",\"shareType\":\"NOT_SHARED\"}],\"currentAuq\":{\"queryName\":\"系统默认查询\",\"entityName\":\"TmlVesselVisitVo\",\"simpleFilter\":{},\"isFuzzy\":false,\"paraFields\":[],\"paraMap\":{},\"displayFields\":[\"id\",\"hbrCvGkey\",\"hcvId\",\"phase\",\"carId\",\"carNameLocal\",\"carVesselStructGkey\",\"expBerthId\",\"expDirection\",\"expStartBollardId\",\"expEndBollardId\",\"actBerthId\",\"actDirection\",\"actStartBollardId\",\"actEndBollardId\",\"expInTmlTime\",\"expWorkStartTime\",\"expWorkEndTime\",\"expOutTmlTime\",\"actInTmlTime\",\"actWorkStartTime\",\"actWorkEndTime\",\"actOutTmlTime\",\"ibVoyage\",\"obVoyage\",\"notes\"],\"selectFields\":[\"gkey\",\"id\",\"hbrCvGkey\",\"hcvId\",\"phase\",\"carId\",\"carNameLocal\",\"carVesselStructGkey\",\"expBerthId\",\"expDirection\",\"expStartBollardId\",\"expEndBollardId\",\"actBerthId\",\"actDirection\",\"actStartBollardId\",\"actEndBollardId\",\"expInTmlTime\",\"expWorkStartTime\",\"expWorkEndTime\",\"expOutTmlTime\",\"actInTmlTime\",\"actWorkStartTime\",\"actWorkEndTime\",\"actOutTmlTime\",\"ibVoyage\",\"obVoyage\",\"notes\"],\"orderByFields\":[[\"changed\",\"DESC\"]],\"pageIndex\":1,\"pageSize\":20},\"fieldDefs\":{\"gkey\":{\"fieldName\":\"gkey\",\"fieldKey\":\"entity.TmlVesselVisitVo.gkey\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"id\":{\"fieldName\":\"id\",\"fieldKey\":\"entity.TmlVesselVisitVo.id\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"phase\":{\"fieldName\":\"phase\",\"fieldKey\":\"entity.TmlVesselVisitVo.phase\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":true},\"hbrCvGkey\":{\"fieldName\":\"hbrCvGkey\",\"fieldKey\":\"entity.TmlVesselVisitVo.hbrCvGkey\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"tmlId\":{\"fieldName\":\"tmlId\",\"fieldKey\":\"entity.TmlVesselVisitVo.tmlId\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"expInTmlTime\":{\"fieldName\":\"expInTmlTime\",\"fieldKey\":\"entity.TmlVesselVisitVo.expInTmlTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"expWorkStartTime\":{\"fieldName\":\"expWorkStartTime\",\"fieldKey\":\"entity.TmlVesselVisitVo.expWorkStartTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"expWorkEndTime\":{\"fieldName\":\"expWorkEndTime\",\"fieldKey\":\"entity.TmlVesselVisitVo.expWorkEndTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"expOutTmlTime\":{\"fieldName\":\"expOutTmlTime\",\"fieldKey\":\"entity.TmlVesselVisitVo.expOutTmlTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"expWorkHours\":{\"fieldName\":\"expWorkHours\",\"fieldKey\":\"entity.TmlVesselVisitVo.expWorkHours\",\"fieldType\":\"Float\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"expInTmlHours\":{\"fieldName\":\"expInTmlHours\",\"fieldKey\":\"entity.TmlVesselVisitVo.expInTmlHours\",\"fieldType\":\"Float\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"actInTmlTime\":{\"fieldName\":\"actInTmlTime\",\"fieldKey\":\"entity.TmlVesselVisitVo.actInTmlTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"actWorkStartTime\":{\"fieldName\":\"actWorkStartTime\",\"fieldKey\":\"entity.TmlVesselVisitVo.actWorkStartTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"actWorkEndTime\":{\"fieldName\":\"actWorkEndTime\",\"fieldKey\":\"entity.TmlVesselVisitVo.actWorkEndTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"actOutTmlTime\":{\"fieldName\":\"actOutTmlTime\",\"fieldKey\":\"entity.TmlVesselVisitVo.actOutTmlTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"ibVoyage\":{\"fieldName\":\"ibVoyage\",\"fieldKey\":\"entity.TmlVesselVisitVo.ibVoyage\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"obVoyage\":{\"fieldName\":\"obVoyage\",\"fieldKey\":\"entity.TmlVesselVisitVo.obVoyage\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"expBerthId\":{\"fieldName\":\"expBerthId\",\"fieldKey\":\"entity.TmlVesselVisitVo.expBerthId\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"expDirection\":{\"fieldName\":\"expDirection\",\"fieldKey\":\"entity.TmlVesselVisitVo.expDirection\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":true},\"expStartBollardId\":{\"fieldName\":\"expStartBollardId\",\"fieldKey\":\"entity.TmlVesselVisitVo.expStartBollardId\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"expEndBollardId\":{\"fieldName\":\"expEndBollardId\",\"fieldKey\":\"entity.TmlVesselVisitVo.expEndBollardId\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"actBerthId\":{\"fieldName\":\"actBerthId\",\"fieldKey\":\"entity.TmlVesselVisitVo.actBerthId\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"actDirection\":{\"fieldName\":\"actDirection\",\"fieldKey\":\"entity.TmlVesselVisitVo.actDirection\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":true},\"actStartBollardId\":{\"fieldName\":\"actStartBollardId\",\"fieldKey\":\"entity.TmlVesselVisitVo.actStartBollardId\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"actEndBollardId\":{\"fieldName\":\"actEndBollardId\",\"fieldKey\":\"entity.TmlVesselVisitVo.actEndBollardId\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"notes\":{\"fieldName\":\"notes\",\"fieldKey\":\"entity.TmlVesselVisitVo.notes\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"creator\":{\"fieldName\":\"creator\",\"fieldKey\":\"entity.TmlVesselVisitVo.creator\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"created\":{\"fieldName\":\"created\",\"fieldKey\":\"entity.TmlVesselVisitVo.created\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"changer\":{\"fieldName\":\"changer\",\"fieldKey\":\"entity.TmlVesselVisitVo.changer\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"changed\":{\"fieldName\":\"changed\",\"fieldKey\":\"entity.TmlVesselVisitVo.changed\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"isDeleted\":{\"fieldName\":\"isDeleted\",\"fieldKey\":\"entity.TmlVesselVisitVo.isDeleted\",\"fieldType\":\"Integer\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":true},\"hcvId\":{\"fieldName\":\"hcvId\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvId\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvCarrierGkey\":{\"fieldName\":\"hcvCarrierGkey\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvCarrierGkey\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvLineOpGkey\":{\"fieldName\":\"hcvLineOpGkey\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvLineOpGkey\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvPhase\":{\"fieldName\":\"hcvPhase\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvPhase\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":true},\"hcvIbVoyage\":{\"fieldName\":\"hcvIbVoyage\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvIbVoyage\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvObVoyage\":{\"fieldName\":\"hcvObVoyage\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvObVoyage\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvCarrierRouteCode\":{\"fieldName\":\"hcvCarrierRouteCode\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvCarrierRouteCode\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvPrePortCode\":{\"fieldName\":\"hcvPrePortCode\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvPrePortCode\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvNextPortCode\":{\"fieldName\":\"hcvNextPortCode\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvNextPortCode\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvExpInHbrTime\":{\"fieldName\":\"hcvExpInHbrTime\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvExpInHbrTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvExpOutHbrTime\":{\"fieldName\":\"hcvExpOutHbrTime\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvExpOutHbrTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvExpWorkHours\":{\"fieldName\":\"hcvExpWorkHours\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvExpWorkHours\",\"fieldType\":\"Float\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvExpInHbrHours\":{\"fieldName\":\"hcvExpInHbrHours\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvExpInHbrHours\",\"fieldType\":\"Float\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvActInHbrTime\":{\"fieldName\":\"hcvActInHbrTime\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvActInHbrTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvActOutHbrTime\":{\"fieldName\":\"hcvActOutHbrTime\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvActOutHbrTime\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvInTradeType\":{\"fieldName\":\"hcvInTradeType\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvInTradeType\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvInContactor\":{\"fieldName\":\"hcvInContactor\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvInContactor\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvOutTradeType\":{\"fieldName\":\"hcvOutTradeType\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvOutTradeType\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvOutContactor\":{\"fieldName\":\"hcvOutContactor\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvOutContactor\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvRcmdTmlId\":{\"fieldName\":\"hcvRcmdTmlId\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvRcmdTmlId\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"hcvNotes\":{\"fieldName\":\"hcvNotes\",\"fieldKey\":\"entity.TmlVesselVisitVo.hcvNotes\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"carId\":{\"fieldName\":\"carId\",\"fieldKey\":\"entity.TmlVesselVisitVo.carId\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"carNameOrigin\":{\"fieldName\":\"carNameOrigin\",\"fieldKey\":\"entity.TmlVesselVisitVo.carNameOrigin\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"carNameLocal\":{\"fieldName\":\"carNameLocal\",\"fieldKey\":\"entity.TmlVesselVisitVo.carNameLocal\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"carType\":{\"fieldName\":\"carType\",\"fieldKey\":\"entity.TmlVesselVisitVo.carType\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":true},\"carCountryCode\":{\"fieldName\":\"carCountryCode\",\"fieldKey\":\"entity.TmlVesselVisitVo.carCountryCode\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":false},\"carContact\":{\"fieldName\":\"carContact\",\"fieldKey\":\"entity.TmlVesselVisitVo.carContact\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"carStatus\":{\"fieldName\":\"carStatus\",\"fieldKey\":\"entity.TmlVesselVisitVo.carStatus\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":true},\"carOwnerGkey\":{\"fieldName\":\"carOwnerGkey\",\"fieldKey\":\"entity.TmlVesselVisitVo.carOwnerGkey\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"carExpireDate\":{\"fieldName\":\"carExpireDate\",\"fieldKey\":\"entity.TmlVesselVisitVo.carExpireDate\",\"fieldType\":\"Date\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"carVesselMmsi\":{\"fieldName\":\"carVesselMmsi\",\"fieldKey\":\"entity.TmlVesselVisitVo.carVesselMmsi\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"carVesselLloyds\":{\"fieldName\":\"carVesselLloyds\",\"fieldKey\":\"entity.TmlVesselVisitVo.carVesselLloyds\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"carVesselCallSign\":{\"fieldName\":\"carVesselCallSign\",\"fieldKey\":\"entity.TmlVesselVisitVo.carVesselCallSign\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"carVesselStructGkey\":{\"fieldName\":\"carVesselStructGkey\",\"fieldKey\":\"entity.TmlVesselVisitVo.carVesselStructGkey\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":true,\"hasChecker\":false,\"valueTranslate\":false},\"carNotes\":{\"fieldName\":\"carNotes\",\"fieldKey\":\"entity.TmlVesselVisitVo.carNotes\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"loId\":{\"fieldName\":\"loId\",\"fieldKey\":\"entity.TmlVesselVisitVo.loId\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"loNameOrigin\":{\"fieldName\":\"loNameOrigin\",\"fieldKey\":\"entity.TmlVesselVisitVo.loNameOrigin\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"loNameLocal\":{\"fieldName\":\"loNameLocal\",\"fieldKey\":\"entity.TmlVesselVisitVo.loNameLocal\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"loContact\":{\"fieldName\":\"loContact\",\"fieldKey\":\"entity.TmlVesselVisitVo.loContact\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false},\"loNotes\":{\"fieldName\":\"loNotes\",\"fieldKey\":\"entity.TmlVesselVisitVo.loNotes\",\"fieldType\":\"String\",\"isObject\":false,\"canBeOrdered\":true,\"hasSelector\":false,\"hasChecker\":false,\"valueTranslate\":false}},\"trans\":{\"entity.TmlVesselVisitVo.gkey\":\"主键\",\"entity.TmlVesselVisitVo.id\":\"编号\",\"entity.TmlVesselVisitVo.phase\":\"阶段状态\",\"entity.TmlVesselVisitVo.phase.V_CREATED\":\"航次已创建\",\"entity.TmlVesselVisitVo.phase.V_BERTHING\":\"船舶靠泊中\",\"entity.TmlVesselVisitVo.phase.V_ON_BERTH\":\"船舶已在泊\",\"entity.TmlVesselVisitVo.phase.V_WORKING\":\"船舶作业中\",\"entity.TmlVesselVisitVo.phase.V_COMPLETED\":\"船舶已完工\",\"entity.TmlVesselVisitVo.phase.V_UNBERTHING\":\"船舶离泊中\",\"entity.TmlVesselVisitVo.phase.V_DEPARTED\":\"船舶已离开\",\"entity.TmlVesselVisitVo.phase.V_CANCELED\":\"航次已取消\",\"entity.TmlVesselVisitVo.phase.V_CLOSED\":\"航次已关闭\",\"entity.TmlVesselVisitVo.hbrCvGkey\":\"港区访问记录主键\",\"entity.TmlVesselVisitVo.tmlId\":\"作业区编号\",\"entity.TmlVesselVisitVo.expInTmlTime\":\"计划到区时间\",\"entity.TmlVesselVisitVo.expWorkStartTime\":\"计划开工时间\",\"entity.TmlVesselVisitVo.expWorkEndTime\":\"计划完工时间\",\"entity.TmlVesselVisitVo.expOutTmlTime\":\"计划出区时间\",\"entity.TmlVesselVisitVo.expWorkHours\":\"计划作业小时\",\"entity.TmlVesselVisitVo.expInTmlHours\":\"计划在区小时\",\"entity.TmlVesselVisitVo.actInTmlTime\":\"实际到区时间\",\"entity.TmlVesselVisitVo.actWorkStartTime\":\"实际开工时间\",\"entity.TmlVesselVisitVo.actWorkEndTime\":\"实际完工时间\",\"entity.TmlVesselVisitVo.actOutTmlTime\":\"实际出区时间\",\"entity.TmlVesselVisitVo.ibVoyage\":\"进作业区航次\",\"entity.TmlVesselVisitVo.obVoyage\":\"出作业区航次\",\"entity.TmlVesselVisitVo.expBerthId\":\"计划泊位编号\",\"entity.TmlVesselVisitVo.expDirection\":\"计划靠泊方向\",\"entity.TmlVesselVisitVo.expDirection.L\":\"左靠\",\"entity.TmlVesselVisitVo.expDirection.R\":\"右靠\",\"entity.TmlVesselVisitVo.expStartBollardId\":\"计划开始缆桩编号\",\"entity.TmlVesselVisitVo.expEndBollardId\":\"计划结束缆桩编号\",\"entity.TmlVesselVisitVo.actBerthId\":\"实际泊位编号\",\"entity.TmlVesselVisitVo.actDirection\":\"实际靠泊方向\",\"entity.TmlVesselVisitVo.actDirection.L\":\"左靠\",\"entity.TmlVesselVisitVo.actDirection.R\":\"右靠\",\"entity.TmlVesselVisitVo.actStartBollardId\":\"实际开始缆桩编号\",\"entity.TmlVesselVisitVo.actEndBollardId\":\"实际结束缆桩编号\",\"entity.TmlVesselVisitVo.notes\":\"备注\",\"entity.TmlVesselVisitVo.creator\":\"创建人\",\"entity.TmlVesselVisitVo.created\":\"创建时间\",\"entity.TmlVesselVisitVo.changer\":\"修改人\",\"entity.TmlVesselVisitVo.changed\":\"修改时间\",\"entity.TmlVesselVisitVo.isDeleted\":\"已删除\",\"entity.TmlVesselVisitVo.isDeleted.1\":\"是\",\"entity.TmlVesselVisitVo.isDeleted.0\":\"否\",\"entity.TmlVesselVisitVo.hcvId\":\"编号(港区)\",\"entity.TmlVesselVisitVo.hcvCarrierGkey\":\"载具主键(港区)\",\"entity.TmlVesselVisitVo.hcvLineOpGkey\":\"承运人主键(港区)\",\"entity.TmlVesselVisitVo.hcvPhase\":\"阶段状态(港区)\",\"entity.TmlVesselVisitVo.hcvPhase.V_CREATED\":\"航次已创建\",\"entity.TmlVesselVisitVo.hcvPhase.V_PREDICTED\":\"航次已预报\",\"entity.TmlVesselVisitVo.hcvPhase.V_CONFIRMED\":\"航次已确报\",\"entity.TmlVesselVisitVo.hcvPhase.V_ANCHORAGE\":\"船舶已在锚\",\"entity.TmlVesselVisitVo.hcvPhase.V_BERTHING\":\"船舶靠泊中\",\"entity.TmlVesselVisitVo.hcvPhase.V_ONBERTH\":\"船舶已在泊\",\"entity.TmlVesselVisitVo.hcvPhase.V_UNBERTHING\":\"船舶离泊中\",\"entity.TmlVesselVisitVo.hcvPhase.V_SHIFTING\":\"船舶移泊中\",\"entity.TmlVesselVisitVo.hcvPhase.V_DEPARTED\":\"船舶已离港\",\"entity.TmlVesselVisitVo.hcvPhase.V_CLOSED\":\"航次已关闭\",\"entity.TmlVesselVisitVo.hcvPhase.V_CANCELED\":\"航次已取消\",\"entity.TmlVesselVisitVo.hcvPhase.R_CREATED\":\"车次已创建\",\"entity.TmlVesselVisitVo.hcvPhase.R_PREDICTED\":\"车次已预报\",\"entity.TmlVesselVisitVo.hcvPhase.R_CONFIRMED\":\"车次已确报\",\"entity.TmlVesselVisitVo.hcvPhase.R_ARRIVING\":\"列车停靠中\",\"entity.TmlVesselVisitVo.hcvPhase.R_ARRIVED\":\"列车已停靠\",\"entity.TmlVesselVisitVo.hcvPhase.R_DEPARTING\":\"列车离开中\",\"entity.TmlVesselVisitVo.hcvPhase.R_SHIFTING\":\"列车转站中\",\"entity.TmlVesselVisitVo.hcvPhase.R_DEPARTED\":\"列车已离港\",\"entity.TmlVesselVisitVo.hcvPhase.R_DISMISSED\":\"列车已解体\",\"entity.TmlVesselVisitVo.hcvPhase.R_CLOSED\":\"车次已关闭\",\"entity.TmlVesselVisitVo.hcvPhase.R_CANCELED\":\"车次已取消\",\"entity.TmlVesselVisitVo.hcvPhase.T_ORDERED\":\"车次已预约\",\"entity.TmlVesselVisitVo.hcvPhase.T_ARRIVED\":\"车辆已到达\",\"entity.TmlVesselVisitVo.hcvPhase.T_DEPARTED\":\"车辆已离港\",\"entity.TmlVesselVisitVo.hcvPhase.T_CLOSED\":\"车次已关闭\",\"entity.TmlVesselVisitVo.hcvPhase.T_CANCELED\":\"车次已关闭\",\"entity.TmlVesselVisitVo.hcvIbVoyage\":\"进港航次(港区)\",\"entity.TmlVesselVisitVo.hcvObVoyage\":\"出港航次(港区)\",\"entity.TmlVesselVisitVo.hcvCarrierRouteCode\":\"路线代码(港区)\",\"entity.TmlVesselVisitVo.hcvPrePortCode\":\"上一港代码(港区)\",\"entity.TmlVesselVisitVo.hcvNextPortCode\":\"下一港代码(港区)\",\"entity.TmlVesselVisitVo.hcvExpInHbrTime\":\"计划进港时间(港区)\",\"entity.TmlVesselVisitVo.hcvExpOutHbrTime\":\"计划离港时间(港区)\",\"entity.TmlVesselVisitVo.hcvExpWorkHours\":\"计划作业小时(港区)\",\"entity.TmlVesselVisitVo.hcvExpInHbrHours\":\"计划在港小时(港区)\",\"entity.TmlVesselVisitVo.hcvActInHbrTime\":\"实际进港时间(港区)\",\"entity.TmlVesselVisitVo.hcvActOutHbrTime\":\"实际离港时间(港区)\",\"entity.TmlVesselVisitVo.hcvInTradeType\":\"进口内外贸(港区)\",\"entity.TmlVesselVisitVo.hcvInContactor\":\"进口联系人(港区)\",\"entity.TmlVesselVisitVo.hcvOutTradeType\":\"出口内外贸(港区)\",\"entity.TmlVesselVisitVo.hcvOutContactor\":\"出口联系人(港区)\",\"entity.TmlVesselVisitVo.hcvRcmdTmlId\":\"首选作业区编号(港区)\",\"entity.TmlVesselVisitVo.hcvNotes\":\"备注(港区)\",\"entity.TmlVesselVisitVo.carId\":\"船舶编号\",\"entity.TmlVesselVisitVo.carNameOrigin\":\"船舶原名称\",\"entity.TmlVesselVisitVo.carNameLocal\":\"船舶本地化名称\",\"entity.TmlVesselVisitVo.carType\":\"船舶载具类型\",\"entity.TmlVesselVisitVo.carType.VESSEL\":\"船舶\",\"entity.TmlVesselVisitVo.carType.RAIL\":\"火车\",\"entity.TmlVesselVisitVo.carType.TRUCK\":\"汽车\",\"entity.TmlVesselVisitVo.carCountryCode\":\"船舶国籍代码\",\"entity.TmlVesselVisitVo.carContact\":\"船舶联系人\",\"entity.TmlVesselVisitVo.carStatus\":\"船舶状态\",\"entity.TmlVesselVisitVo.carStatus.ACTIVE\":\"激活\",\"entity.TmlVesselVisitVo.carStatus.BANNED\":\"禁止\",\"entity.TmlVesselVisitVo.carOwnerGkey\":\"船舶所有人外键\",\"entity.TmlVesselVisitVo.carExpireDate\":\"船舶失效日期\",\"entity.TmlVesselVisitVo.carVesselMmsi\":\"船舶海上移动识别码\",\"entity.TmlVesselVisitVo.carVesselLloyds\":\"船舶劳埃德编号\",\"entity.TmlVesselVisitVo.carVesselCallSign\":\"船舶呼号\",\"entity.TmlVesselVisitVo.carVesselStructGkey\":\"船舶结构主键\",\"entity.TmlVesselVisitVo.carNotes\":\"船舶注释\",\"entity.TmlVesselVisitVo.loId\":\"承运人编号\",\"entity.TmlVesselVisitVo.loNameOrigin\":\"承运人原名称\",\"entity.TmlVesselVisitVo.loNameLocal\":\"承运人名称\",\"entity.TmlVesselVisitVo.loContact\":\"承运人联系方式\",\"entity.TmlVesselVisitVo.loNotes\":\"承运人备注\"}}"
        ThreeResultExtras threeResultExtras = GsonUtil.fromJsonObject(extras, ThreeResultExtras.class)
        auqResult.setExtras(threeResultExtras)
        return auqResult
    }
}
